function [Wtr]=ANN_CONV_W(Wini,Nin,Nhid,Nout,Type)
//******************************************************************
//  Function to convert the weight and bias stored in a matrix or
//  vector form in the other form (vector or matrix form respectively)
//
//  Input arguments of the function :
//        Wini  FOR THE MATRIX FORM : Weight and bias values (2 dimensions Matrix [max(Nhid,M) x max(P+1,Nhid+1) x 2])
//                Wini(1:Nhid,1,1)      are the bias for the hidden neurons
//                Wini(1:Nhid,2:P+1,1)  are the weights for the hidden neurons (P weights for each hidden neuron)
//                Wini(1:M,1,2)         are the bias for the ouput neurons
//                Wini(1:M,2:Nhid+1,2)  are the weights for the ouput neurons (Nhid weights for each output neuron) 
//              FOR THE VECTOR FORM : matrix [NP,1] (NP number of parameters). The vector form is deduced from the matrix form
//              by agregating each lines of W one after another starting by the first layer :
//                Wini(1:Nin+1)                             Bias and weights for first hidden neuron
//                Wini([Nin+1]+1:2*[Nin+1])                 Bias and weights for second hidden neuron
//                ...
//                Wini([Nin+1]*Nhid+1:[Nin+1]*Nhid+[Nhid+1])              Bias and weights for first ouput neuron
//                Wini([Nin+1]*Nhid+[Nhid+1]+1:[Nin+1]*Nhid+2*[Nhid+1])   Bias and weights for second ouput neuron
//                ...
//        Nin   Number of neurons in the input layer
//        Nhid  Number of neurons in the hidden layer
//        Nout  Number of neurons in the output layer
//        Type  Initial type ('matrix' or 'vector') of weight and bias data (the output variable will be of the other type...)
//
//  Output arguments of the function :
//        Wtr   Weight and bias transformed in the vector or matrix form
//        POS   correspondance between matrix indices and vector indices (matrix [NP,3]).
//              POS(i,:) are 3 values locating the ith network parameter in a matrix form :
//                Col 1:  Line indices in the matrix type
//                Col 2:  Column indices in the matrix type
//                Col 3:  Layer indices in the matrix type (1 or 2)

//------ Selection of transformation -------
if Type=='matrix' then
    //------ Validation of input variables -------
        if size(Wini,3)<>2                           then, error('First input variable is not an ANN biais weight matrix'); end
        if size(Wini,1)>max(Nout,Nhid)               then, error('Wrong size of initial weights matrix for the first dimension'); end;
        if size(Wini,2)>max(Nin+1,Nhid+1)            then, error('Wrong size of initial weights matrix for the second dimension'); end;
  
    //------ Conversion to vector form -------
        POS=[];
        index=0;
        for Layer=1:2
          if Layer==1 then Nlayp=Nin;Nlay=Nhid; else Nlayp=Nhid;Nlay=Nout; end  // Determine the number of neuron in the previous leayer (Nlayp) and in the layer (Nlay)
          for Neuron=1:Nlay
            Wtr(index+1:index+Nlayp+1)    = Wini(Neuron,1:Nlayp+1,Layer);
            POS(index+1:index+Nlayp+1,1:3)= [Neuron*ones(Nlayp+1,1),(1:Nlayp+1)',Layer*ones(Nlayp+1,1)];
            index                         = index+Nlayp+1;
          end
       end
       Wtr=Wtr';  
        
elseif  Type=='vector' then
    //------ Validation of input variables -------
        if size(Wini,1)<>(Nin+1)*Nhid+(Nhid+1)*Nout  then, error('Incompatibility between the bias and weight vector and the other input variables'); end
  
    //------ Conversion to matrix form -------
        Wtr=hypermat([max(Nhid,Nout) max(Nin+1,Nhid+1) 2]);
        POS=[];
        index=0;
        for Layer=1:2
          if Layer==1 then Nlayp=Nin;Nlay=Nhid; else Nlayp=Nhid;Nlay=Nout; end  // Determine the number of neuron in the previous leayer (Nlayp) and in the layer (Nlay)
          for Neuron=1:Nlay
            Wtr(Neuron,1:Nlayp+1,Layer)   = Wini(index+1:index+Nlayp+1);
            POS(index+1:index+Nlayp+1,1:3)= [Neuron*ones(Nlayp+1,1),(1:Nlayp+1)',Layer*ones(Nlayp+1,1)];
            index                         = index+Nlayp+1;
          end
       end
else
  error('Conversion type not recognized')

end

endfunction

