function [JE,JJ,normJE,varargout]=ANN_JACOB(IN,W,IN_W,HID_OUT,ERR_OUT)
//******************************************************************
//  Function to calculate the jacobian performance vector of an feed forward artifical neural
//  network. The activation function of the hidden layer is the hyperbolic tangent 
//  and the identity function for the output layer.
//  This function is used in the function 'LMBR' for the training of feed-forward 
//  neural network with Levenberg-Marquadt algorithm under bayesian regulation.
//
//  Input arguments of the function :
//        IN    Input data (Matrix [PxN] where P is the number of input neurons and N the number of input patterns)
//        W     Weight and bias values (2 dimensions Matrix [max(Nhid,M) x max(P+1,Nhid+1) x 2]). 
//              W(1:Nhid,1,1)      are the bias for the hidden neurons
//              W(1:Nhid,2:P+1,1)  are the weights for the hidden neurons (P weights for each hidden neuron)
//              W(1:M,1,2)         are the bias for the ouput neurons
//              W(1:M,2:Nhid+1,2)  are the weights for the ouput neurons (Nhid weights for each output neuron) 
//        IN_W  Weighted inputs
//        HID_OUT Ouputs from the hidden layer
//        ERR_OUT Errors (target-network ouput)
//
//  Output arguments of the function :
//        JE      Jacobian times errors (Matrix [N_PARxNB_OUT] where NB_PAR is the number of parameters in the network
//        JJ      Transposed jacobian times jacobian (Matrix [N_PARxNB_PAR])
//
//  
//------ Initialisation -------
    NB_PT     = size(IN,2);                                                     // Number of patterns
    NB_IN     = size(IN,1);                                                     // Number of inputs
    NB_OUT    = size(ERR_OUT,1);                                                // Number of outputs
    NB_HID    = size(HID_OUT,1);                                                // Number of hidden neurons
    PREMPAR_H2Olayer  = (NB_IN+1)*NB_HID;                                       // Number of the first parameter for the hidden-output layer connection in the vector form
    
//------ Signal expansion -------
    ii_OUT  = [(1:NB_OUT)'.*.ones(NB_PT,1) ones(NB_OUT,1).*.(1:NB_PT)'];        // Expands the errors (matrix [NB_OUT x N]) in one vector (vector [NB_OUT.N x 1])  
    ERR_X   = diag(ERR_OUT(ii_OUT(:,1),ii_OUT(:,2)));                           // The expansion treats the errors line by line (each line is a vector of N elements)

//------ MAIN PART OF THE FUNCTION : Jacobian calculation -------

    // Jacobian for the parameters linking the hidden layer to the output layer
    // Activation function = identity !!!    
    for i=1:NB_OUT
        JX(PREMPAR_H2Olayer+(i-1)*(NB_HID+1)+1,:) = ...                         // Jacobian for the biases
            [zeros(1,NB_PT*(i-1)) -1*ones(1,NB_PT) zeros(1,NB_PT*(NB_OUT-i))];        

        for j=2:NB_HID+1
            JX(PREMPAR_H2Olayer+(i-1)*(NB_HID+1)+j,:) = ...                     // Jacobian for the weights
                [zeros(1,NB_PT*(i-1)) -1*HID_OUT(j-1,:) zeros(1,NB_PT*(NB_OUT-i))];
        end
    end
    // Jacobian for the parameters linking the input layer to the hidden layer
    // Activation function = tanh !!!    
    for i=1:NB_HID
        temp  = -1 * diag(W(1:NB_OUT,i+1,2)).*(ones(NB_OUT,1)*(1-tanh(IN_W(i,:)).^2));    // Temporary matrix helpful for the calculation
        
        JX((i-1)*(NB_IN+1)+1,:) = diag(temp(ii_OUT(:,1),ii_OUT(:,2)))';         // Jacobian for the biases (expansion of the temp matrix)

        for j=2:NB_IN+1
            temp2 = temp.*(IN(j-1,:)*ones(NB_OUT,1));                            // Temporary matrix helpful for the calculation
            JX((i-1)*(NB_IN+1)+j,:) = diag(temp2(ii_OUT(:,1),ii_OUT(:,2)))';    // Jacobian for the weights (expansion of the temp2 matrix)
        end
    end  
    
//------ Output variables---------------
    JJ      = JX*JX';
    JE      = JX*ERR_X;                                                         
    normJE  = (JE'*JE).^2;                                                      // Gradient norm

    [lhs,rhs]=argn();
    if lhs>=3 then, varargout(1)=JX; end


endfunction
