function [W,OUT,RMSE,varargout]=ANN_LMBR(IN,TARG,Nhid,Wini,varargin)
//******************************************************************
//  Function to train a feed-forward artificial neural network with 
//  one hidden layer. The activation function of the hidden layer is the
//  hyperbolic tangent and the identity function for the output layer.
//
//  The objective function to be minimized is the Sum of Squared Errors (SSE). 
//
//  The training algorithm is Levenberg-Marquadt algorithm with bayesian regulation.
//  (see MacKay, Neural Computation, vol. 4, no. 3, 1992, pp. 415-447.
//  Foresee and Hagan, Proceedings of the International Joint Conference on Neural Networks, June, 1997.)
//
//  Input arguments of the function :
//        IN    Input data (matrix [PxN] where P is the number of input neurons and N the number of input patterns)
//        TARG  Target data (matrix [MxN] where M is the number of ouput neurons and N the number of input patterns)
//        Nhid  Number of neurons in the hidden layer
//        Wini  Initial weight and bias values (2 dimensions Matrix [max(Nhid,M) x max(P+1,Nhid+1) x 2]). 
//              Wini(1:Nhid,1,1)      are the bias for the hidden neurons
//              Wini(1:Nhid,2:P+1,1)  are the weights for the hidden neurons (P weights for each hidden neuron)
//              Wini(1:M,1,2)         are the bias for the ouput neurons
//              Wini(1:M,2:Nhid+1,2)  are the weights for the ouput neurons (Nhid weights for each output neuron) 
//        {opt. param. 1}  EPOCH      Number of traning epoch
//        {opt. param. 2}  EpochShow  Epochs between displays (NaN for no displays)
//        {opt. param. 3}  GraphFin   Display an intermediate plot of the results or not
//
//
//  Output arguments of the function :
//        W     Final weight and biais values (same matrix structure than Wini)
//        OUT   network outputs (Matrix [MxN] where M is the number of ouput neurons and N the number of input patterns)
//
//  Description of the main internal variable of the function
//        SSE   Sum of squared errors
//        GAMK  Effective number of parameters. If GAMK << NB_PT then the network is over parametrised
//        SSX   Sum of squared weigths
//        PERF  balanced performance between SSE(times BETA) and SSX(times ALPHA)
//        IN_W  Weighted inputs
//        HID_OUT Ouputs from the hidden layer

//------ Validation of input variables -------
    if size(IN,2)<>size(TARG,2)    then, error('Input and target data do not have the same number of patterns'); end;
    if size(Wini,1)>max(size(TARG,1),Nhid) then, error('Wrong size of initial weights matrix for the first dimension'); end;    
    if size(Wini,2)>max(size(IN,1)+1,Nhid+1) then, error('Wrong size of initial weights matrix for the second dimension'); end;
    if find(isnan(IN))<>[] | find(isnan(TARG))<>[] then, error('NaN values in IN or TARG data'); end;

//------ Algorithm parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined

    EPOCH     = 30;if rhs >= 5  then, if varargin(1)<>'def' then, EPOCH  = max(2,varargin(1));end;end // Number of epochs (>=2)
    EpochShow = 0;if rhs >= 6  then, if varargin(2)<>'def' then, EpochShow  = varargin(2);end;end;  // Epochs between displays 
                                                                                                    // (0 for no displays)
    GraphFin  = %T;if rhs >= 7  then, if varargin(3)<>'def' then, GraphFin  = varargin(3);end;end;  // Graph plot of the 
                                                                                                    // results
    MaxTime   = %inf;if rhs >= 8  then, if varargin(4)<>'def' then, MaxTime = varargin(4);end;end;  // Maximum time to train 
                                                                                                    // in seconds    
    MU        = 0.005;if rhs >= 9  then, if varargin(5)<>'def' then, mu   = varargin(5); end;end;// Marquardt adjustment param.
    mu_dec    = 0.1;if rhs >= 10 then, if varargin(6)<>'def' then, mu_dec = varargin(6); end;end;   // Decrease factor for mu  
    mu_inc    = 10;if rhs >= 11 then, if varargin(7)<>'def' then, mu_inc  = varargin(7); end;end;   // Increase factor for mu
    mu_max    = 1D10;if rhs >= 12 then, if varargin(8)<>'def' then, mu_max= varargin(8); end;end;   // Maximum value for mu
    max_fail  = 5;if rhs >= 13 then, if varargin(9)<>'def' then, max_fail = varargin(9); end;end;   // Maximum validation
                                                                                                    // failures
    min_grad  = 1D-10; if rhs >= 14 then, if min_grad<>'def' then, min_grad  = varargin(10); end;end;// Minimum performance
                                                                                                     // gradient

//------ Initialisation -------
    W         = Wini;
    NB_PT     = size(IN,2);                                                     // Number of patterns
    NB_OUT    = size(TARG,1);                                                   // Number of ouputs
    NB_ERR    = NB_PT*NB_OUT;                                                   // Number of errors    
    X         = ANN_CONV_W(Wini,size(IN,1),Nhid,NB_OUT,'matrix');               // Gets the parameter vector and 
                                                                                //   the correspondances between 
                                                                                //   the matrix and the vector form
                                                                                
    NB_PAR    = size(X,1);                                                      // Number of parameters
    II        = sparse([(1:NB_PAR)' (1:NB_PAR)'],ones(NB_PAR,1));               // Identity matrix

    [OUT,IN_W,HID_OUT] =  ANN_SIM(IN,Nhid,NB_OUT,Wini);                         // First simulation, results
    ERR_OUT   = TARG-OUT;                                                       // Errors

    // Regularization parameters
    SSE(1)    = sum((TARG-OUT).^2);                                             // Initial sum of squared errors
    GAMK(1)   = NB_PAR;                                                         // Initial effective number of parameters
    BETA      = (NB_ERR-GAMK(1))/(2*SSE(1));
    if BETA<=0 then BETA=1; end;
    SSX(1)    = X'*X;
    ALPHA     = GAMK(1)/(2*SSX(1));
    PERF (1)  = BETA*SSE(1) + ALPHA*SSX(1);                                     // Initial performance
    
    // Loop control
    LoopStop  = %f;

//------ MAIN PART OF THE FUNCTION : Loop over epochs -------
    tic();
    
    //------ performance display for the first epoch
    RMSE  = (sum((TARG-OUT).^2)/size(TARG,2)).^0.5;
    if EpochShow>0 then, mprintf('\n\nEpoch=%3d\tRMSE=%6.2f\tGamk=%6.2f/%3d\tMU=%6.2f\tALPHA=%6.2f'+ ..
                  '\tBETA=%6.2f\n\t\tPERF=%6.2f\tSSE=%6.2f\tSSX=%6.2f\n',..
                  1,RMSE,GAMK(1),size(X,1),MU,ALPHA,BETA,PERF(1),SSE(1),SSX(1)); end;
    
    for e=2:EPOCH
        //------ Jacobian --------
        [JE,JJ,NORMGX]=ANN_JACOB(IN,W,IN_W,HID_OUT,ERR_OUT);
        
        //------ Increase MU until errors decrease
        MUloop=1;
        while (MU<=mu_max)
            dX    = -(BETA*JJ+II*(MU+ALPHA))\(BETA*JE+ALPHA*X);
            X2    = X+dX;
            SSX2  = X2'*X2;
            W2    = ANN_CONV_W(X2,size(IN,1),Nhid,size(TARG,1),'vector');

            [OUT,IN_W,HID_OUT] =  ANN_SIM(IN,Nhid,NB_OUT,W2);                     // Simulation with updated parameters
            ERR_OUT            =  TARG-OUT;                                       // updated errors
            
            SSE2  = sum((TARG-OUT).^2);                                           // Calculates the sum of squared errors
            PERF2 = BETA*SSE2 + ALPHA*SSX2;
            
            //mprintf('\n\t\t\tMU optimisation loop N=%3d\tdelta PERF=%7.3f\tMU=%6.2f',MUloop,PERF(e-1)-PERF2,MU);  
            // Follow the MU values

            if PERF2<PERF(e-1) then                                               // If the performance is deacreased
                SSX(e)=SSX2;SSE(e)=SSE2;W=W2;X=X2;                                // Validate changes
                MU  = MU*mu_dec;                                                  // Decrease MU
                MU  = max(MU,1D-20);                                              // Minimal value for MU 
                break
            end
            
            MU    = MU*mu_inc;                                                    // MU is increased if no performance
                                                                                  // improvement can be reached
            MUloop= MUloop+1;
        end
        
        //------ Update regularization parameters ----------
        if MU<=mu_max then
            GAMK(e) = NB_PAR-ALPHA*trace(inv(BETA*JJ+II*ALPHA));                  // Effective number of parameters
            // Attention, Calcul de GAMK avec le jacobien sans mise  jour de dX
            // ncessit de recalculer le Jacobien avant cette ligne !! 
            
            ALPHA   = GAMK(e)/(2*SSX(e));
            BETA    = (NB_ERR-GAMK(e))/(2*SSE(e));
            PERF(e) = BETA*SSE(e)+ALPHA*SSX(e);                                   // Final performance for the epoch e
        end
        
        //------ Stopping criteria --------        
        if toc()>MaxTime    then, mprintf('\nMaximum time elapsed, Stop at epoch n%3d/%3d\n',e,EPOCH);       LoopStop=%t; end;
        if MU>mu_max        then, mprintf('\nMaximum MU reached, Stop at epoch n%3d/%3d\n',e-1,EPOCH);       LoopStop=%t; end;        
        if NORMGX<min_grad  then, mprintf('\nMinimum gradient reached, Stop at epoch n%3d/%3d\n',e,EPOCH);   LoopStop=%t; end;    

        //------ performance display if required
        RMSE  = (sum((TARG-OUT).^2)/size(TARG,2)).^0.5;
        if e-int(e/max(EpochShow,1))*EpochShow==0 & ~LoopStop & EpochShow>0 then, ...
                  mprintf('\n\n Epoch=%3d\tRMSE=%6.2f\tGamk=%6.2f/%3d\tMU=%6.2f\tALPHA=%6.2f'+..
                  '\tBETA=%6.2f\n\t\tPERF=%6.2f\tSSE=%6.2f\tSSX=%6.2f\tNormGX=%6.2f\n',...
                  e,RMSE,GAMK(e),size(X,1),MU,ALPHA,BETA,PERF(e),SSE(e),SSX(e),NORMGX); end;
                       
        //------ Graph plot -------------------------
        if GraphFin then
            xset('window',999);fig=get('current_figure');fig.figure_size=[300,700];
            subplot(3,1,1),plot2d(1:size(SSE,'*'),SSE,style=2),xtitle('Sum of squared errors','Epochs','SSE')
                gr=gca();gr.margins=0.15*ones(1,4);
            subplot(3,1,2),plot2d(1:size(SSX,'*'),SSX,style=3),xtitle('Sum of squared weights','Epochs','SSX')
                gr=gca();gr.margins=0.15*ones(1,4);
            subplot(3,1,3),plot2d(1:size(GAMK,'*'),GAMK,style=4),plot2d([1;e],[NB_PAR;NB_PAR],style=5),
                           xtitle('Number of well determined parameters','Epochs','GAMK'),
                           if e==EPOCH then, xstring(e/5,NB_PAR*0.9,'Total number of parameters'); end
                gr=gca();gr.margins=0.15*ones(1,4);
        end
        
        
        //------ End of Epoch loop -------------------------
        if LoopStop | e==EPOCH then
            //------ Output variables---------------
            RMSE  = (sum((TARG-OUT).^2)/size(TARG,2)).^0.5;
            if lhs>=4 then, varargout(1)=SSE;     end
            if lhs>=5 then, varargout(2)=GAMK;    end
            if lhs>=6 then, varargout(3)=SSX;     end
            if lhs>=7 then, varargout(4)=PERF;    end
            if lhs>=8 then, varargout(5)=IN_W;    end
            if lhs>=9 then, varargout(6)=HID_OUT; end
            return
        end
    
    end // loop over epochs

endfunction


 
