function [DATA_final,DATA_mean,DATA_std]=ANN_NORM(DATA,varargin)
//******************************************************************
//  Function to normalise data to train a feed forward network
//
//  Input arguments of the function :
//        DATA  Input data (matrix [PxN] where P is the number of neurons and N the number of input patterns)
//        {opt. param 1}  OBJ   Objective of the normalisation ('raw2norm' or 'norm2raw')
//        {opt. param 2}  MULT  Multiplicator of standard deviation to use for the normalisation process (default = 1)
//                              This parameter can be a matrix of size [Px1], with one multiplicator for each inputs
//        {opt. param 3}  MEAN  Means to use in the normalisation process (matrix [Px1])
//        {opt. param 4}  STD   Standard deviations to use in the normalisation process (matrix [Px1])
//
//  Output arguments of the function :
//        DATA_final  DATA transformed by the normaliation of the 'de'normalisation process
//        DATA_mean   Means used to normalise DATA (vector [Px1])
//        DATA_std    Standard deviations used to normalise DATA (vector [Px1])
//
//------ Normalisation parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined


    OBJ         = 'raw2norm';   if rhs >= 2 then, OBJ   = varargin(1); end;
    
    MULT        = 1;            if rhs >= 3 then, MULT  = varargin(2); end;
     if ~(size(MULT,1)==1|size(MULT,1)==size(DATA,1)) | size(MULT,2)<>1 then, error('Wrong size of the multiplicator vector'); end

    NORM_PARAM  = %F;     
    if rhs >= 4 then
        if rhs == 4 then, error('No standard deviation values, only means values!'); end
        NORM_PARAM  = %T;
        MEAN  = varargin(3);
        STD   = varargin(4);
        if size(MEAN,2)<>1 | size(STD,2)<>1 then, error('Wrong size of the means and standard deviation vectors'); end
    end

//------ Core part of the function -----
    // From raw values to normalised ones
    if OBJ=='raw2norm' then
        if NORM_PARAM then
            DATA_mean = MEAN;
            DATA_std  = STD;
        else
            DATA_mean = mean(DATA,'c');
            DATA_std  = st_deviation(DATA,'c');
        end

        DATA_final = (DATA-DATA_mean*ones(1,size(DATA,2)))./(diag(MULT)*DATA_std*ones(1,size(DATA,2)));

    // From normalised values to raw ones
    elseif OBJ=='norm2raw' then
        if NORM_PARAM then
             DATA_mean  = MEAN;
             DATA_std   = STD;
             DATA_final = DATA.*(diag(MULT)*DATA_std*ones(1,size(DATA,2)))+DATA_mean*ones(1,size(DATA,2));
        else
            error('No means and standard deviation values!');
        end
    
    else
        error('No proper direction of the normalisation');
    end

endfunction
