function [CI,varargout]=ANN_OIPDERIV(IN,Nhid,W)
//******************************************************************
//  Function to calculate the partial derivative of outputs with respects to inputs of a feed-forward artificial neural 
//  network with one hidden layer and one output variable. The activation function of the hidden layer is the
//  hyperbolic tangent and the identity function for the output layer.
//
//  Input arguments of the function :
//        IN    Input data (matrix [PxN] where P is the number of input neurons and N the number of input patterns)
//        Nhid  Number of neurons in the hidden layer
//        W     Weight and bias values (2 dimensions Matrix [max(Nhid,M) x max(P+1,Nhid+1) x 2]). 
//              W(1:Nhid,1,1)      are the bias for the hidden neurons
//              W(1:Nhid,2:P+1,1)  are the weights for the hidden neurons (P weights for each hidden neuron)
//              W(1:M,1,2)         are the bias for the ouput neurons
//              W(1:M,2:Nhid+1,2)  are the weights for the ouput neurons (Nhid weights for each output neuron) 
//
//  Output arguments of the function :
//        CI    Coefficient of importance of each input (matrix [size(IN,1) x 1]) = sum of square of partial derivative series
//        Arg1  {PD} Partial derivative (matrix [size(IN,1) x N]) 
//
//------ Validation of input variables -------
    if size(W,3)<>2                           then, error('Third input variable is not an ANN weight matrix'); end      
    if size(W,1)>max(1,Nhid)                  then, error('Wrong size of weights matrix for the first dimension'); end;  // Nout = 1;  
    if size(W,2)>max(size(IN,1)+1,Nhid+1)     then, error('Wrong size of weights matrix for the second dimension'); end;  

//------ Initialisations -------
    P       = size(IN,1);                                   // Number of input neurons
    N       = size(IN,2);                                   // Number of input patterns
    
//------ Weighted values from input layer --------
    IN_W    = W(1:Nhid,2:P+1,1)*IN+W(1:Nhid,1,1)*ones(1,N);

//------ Output from the hidden layer --------
    HID_OUT = tanh(IN_W);                                           // Tanh = activation function of the hidden layer

//------ Calculation of partial derivatives --------
    for i=1:P
        PD(i,:)  = sum(diag(W(1,2:Nhid+1,2))*diag(W(1:Nhid,i+1,1))*(1-HID_OUT.^2),'r');
    end
    
    CI  = sum(PD.^2,'c');

//------ Output variables---------------
    [lhs,rhs]=argn();
    if lhs>=2 then, varargout(1)=PD; end
    
endfunction
