function [OUT,varargout]=ANN_SIM(IN,Nhid,Nout,W)
//******************************************************************
//  Function to simulate the outputs of a feed-forward artificial neural 
//  network with one hidden layer. The activation function of the hidden layer is the
//  hyperbolic tangent and the identity function for the output layer.
//
//  Input arguments of the function :
//        IN    Input data (matrix [PxN] where P is the number of input neurons and N the number of input patterns)
//        Nhid  Number of neurons in the hidden layer
//        Nout  Number of neurons in the ouput layer
//        W     Weight and bias values (2 dimensions Matrix [max(Nhid,M) x max(P+1,Nhid+1) x 2]). 
//              W(1:Nhid,1,1)      are the bias for the hidden neurons
//              W(1:Nhid,2:P+1,1)  are the weights for the hidden neurons (P weights for each hidden neuron)
//              W(1:M,1,2)         are the bias for the ouput neurons
//              W(1:M,2:Nhid+1,2)  are the weights for the ouput neurons (Nhid weights for each output neuron) 
//
//  Output arguments of the function :
//        OUT   network outputs (Matrix [NoutxN])
//        Arg1  {IN_W} weighted input from the input layer (matrix [Nhid x N]) 
//                Note - weighted output from the hidden layer is equal to HID_OUT because of the identity activation function
//        Arg2  {HID_OUT} outputs from the hidden layer (matrix [Nhid x N]) 
//
//------ Validation of input variables -------
    if size(W,3)<>2                           then, error('Fourth input variable is not an ANN weight matrix'); end      
    if size(W,1)>max(Nout,Nhid)               then, error('Wrong size of initial weights matrix for the first dimension'); end;    
    if size(W,2)>max(size(IN,1)+1,Nhid+1)     then, error('Wrong size of initial weights matrix for the second dimension'); end;  

//------ Initialisations -------
    P       = size(IN,1);                                   // Number of input neurons
    N       = size(IN,2);                                   // Number of input patterns

//------ Weighted values from input layer --------
    IN_W    = W(1:Nhid,2:P+1,1)*IN+W(1:Nhid,1,1)*ones(1,N);

//------ Output from the hidden layer --------
    HID_OUT = tanh(IN_W);                                           // Tanh = activation function of the hidden layer
    
//------ Output from the hidden layer --------
    OUT     = W(1:Nout,2:Nhid+1,2)*HID_OUT+W(1:Nout,1,2)*ones(1,N); // Id = activation function of the output layer

//------ Output variables---------------
    [lhs,rhs]=argn();
    if lhs>=2 then, varargout(1)=IN_W; end
    if lhs>=3 then, varargout(2)=HID_OUT; end
    
endfunction
