function [Crit]=CRIT(OBS,CALC)
//******************************************************************
//  Function to calculate performance criteria 
//  The function calculates different criteria evaluating the distance 
//  between an observed time series and a calculated one. 
//
//  Input arguments of the function :
//        OBS  Observed data (matrix [Mx1] where M is the number of time steps)
//        CALC Calculated data (matrix [Mx1] where M is the number of time steps)
//
//  Output arguments of the function :
//        Crit Vector of calculated criteria :
//              - Crit(1) = RMSE
//              - Crit(2) = NASH
//              - Crit(3) = C2M [ =NASH(2-NASH) ]
//              - Crit(4) = Relative Mean error mean(CALC-OBS) / mean(OBS)
//              - Crit(5) = Absolute error exceeded 20% of time

//------ Validation of input variables -------
    if size(OBS,2)<>1&size(CALC,2)<>1 then, error('OBS or CALC are not column vector'); end;
    if size(OBS,1)<>size(CALC,1)      then, error('OBS and CALC do not have the same size'); end;

//------ CORE function code -------
    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined
    NBARGFIX = 2;
    
    parCalcLag=[0,0]; // Parameterones to calculate the optimal lag
    if rhs >= NBARGFIX+1 then, if varargin(1)<>'def' then,parCalcLag  = varargin(1); end;end;

//------ CORE function code -------

  // RMSE
  Crit(1)=sqrt(sum((OBS-CALC).^2)/size(OBS,1));

  // Nash Criteria
  Crit(2)=%nan;
  if sum((OBS-mean(OBS)).^2)<>0 then, Crit(2)=1-sum((OBS-CALC).^2)/sum((OBS-mean(OBS)).^2); end

  // C2M 
  Crit(3)=%nan;
  if ~isnan(Crit(2)) then, Crit(3)=Crit(2)/(2-Crit(2)); end
  
  // Mean error
  if mean(OBS)<>0 then,
    Crit(4)=mean(CALC-OBS)/mean(OBS);
  else
    Crit(4)=mean(CALC-OBS);
  end

  // Error 80%
  temp = perctl(abs(CALC-OBS),80);
  Crit(5)=temp(1);


endfunction


