function [figNum]=GRAPH_CORREL(DATA,varargin)
//******************************************************************
//  Function to generate graphs showing correlations in a set of
//  variables
//
//  Input arguments of the function :
//        DATA           Data to be plotted, matrix [NxP] with N = nb of value and P = Nb of variables
//        {opt. param. 1}  LEG	   Name of the variables (matrix of string [1xP]
//        {opt. param. 2}  PAIRS  List of pairs of variable to plot (matrix [2xM] )
//        {opt. param. 3}  MINMAX Min and max to plot each variables, matrix [Px2]: [[min VAR] [max VAR]]
//        {opt. param. 4}  FSIZE  Size of figure window, matrix [1x2]
//        {opt. param. 5}  PSIZE  Size of the points
//        {opt. param. 6}  LAB    Draw labels ? (%f or vector of N strings), default = %f
//        {opt. param. 6}  SCL    type of scale plots: 
//                                nn = normal, ln = xLog, yNormal, nl = xLog, yNormal, ll = xLog, yLog
//
//  Output arguments of the function :
//        FichGraph     List of generated files
//******************************************************************************************

//------ Validation of input variables -------
    Nserie = size(DATA,2); // Number of time series
    Nval = size(DATA,1); // Number of values

//------ Optional parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined
    NBARGFIX  = 1;

    LEG = [];for s=1:Nserie, LEG=[LEG 'V'+string(s)]; end;
    if rhs >= NBARGFIX+1 then, if varargin(1)<>'def' then, LEG  = varargin(1); end; end;
    if size(LEG,2)<>Nserie then,  error('Wrong size of LEG (number of time series)'); end;
    LEG = string(LEG); // String conversion
    
    PAIRS = COMBIN(list(1:Nserie,1:Nserie));PAIRS = PAIRS(:,find(PAIRS(1,:)<PAIRS(2,:)));
    if rhs >= NBARGFIX+2 then, if varargin(2)<>'def' then, PAIRS  = varargin(2); end; end;
    if find(PAIRS>Nserie)<>[] then, error('Wrong value in PAIRS'); end;

    MINMAX = [min(DATA,'r')' max(DATA,'r')'];
    MINMAX(find(MINMAX(:,2)-MINMAX(:,1)<=0),2) = MINMAX(find(MINMAX(:,2)-MINMAX(:,1)<=0),1)+1;
    if rhs >= NBARGFIX+3 then, if varargin(3)<>'def' then, MINMAX  = varargin(3); end; end
    if find(MINMAX(:,2)-MINMAX(:,1)<=0)<>[] then,  ...
        error('Wrong values in MINMAX, min should be lower than max'); end;
    if size(MINMAX,1)<>Nserie | size(MINMAX,2)<>2 then,  error('Wrong size of MINMAX'); end;
        
    FSIZE = [1000,1000];
    if rhs >= NBARGFIX+4 then, if varargin(4)<>'def' then, FSIZE  = varargin(4); end; end;
    if size(FSIZE,1)<>1 | size(FSIZE,2)<>2 then, error('Wrong size of FSIZE'); end;

    PSIZE = 5;
    if rhs >= NBARGFIX+5 then, PSIZE  = varargin(5); end

    LAB = 0;
    if rhs >= NBARGFIX+6 then, LAB  = varargin(6); end
    if size(LAB,'*')<>size(DATA,1) & size(LAB,'*')==1 then, okLab = %f; else, okLab = %t; end;
    if size(LAB,'*')<>size(DATA,1) & okLab then, error('Wrong number of labels in LAB, should be '+string(size(DATA,1))); end;
      
    SCL = part(msprintf('nn%d\n',(1:size(PAIRS,2))'),1:2);
    if rhs >= NBARGFIX+7 then, SCL  = varargin(7); end
    if size(SCL,'*')<>size(PAIRS,2) then, error('Wrong value in SCL'); end;
      
 //------ CODE CODE -------
  FichGraph=[];
  
  // Number of graph to draw
  NBG = size(PAIRS,2);
  NBXG = int(sqrt(NBG));
  NBYG = 1; while NBYG*NBXG<NBG, NBYG=NBYG+1; end;

  // Dimensions of the figure
  set("figure_style","new");clf();
  fig=get("current_figure");
  fig.figure_size=FSIZE;
  figNum = fig.figure_id;

  for g=1:NBG // Loop on pairs of variables
      
      R = [MINMAX(PAIRS(1,g),1),MINMAX(PAIRS(2,g),1),MINMAX(PAIRS(1,g),2),MINMAX(PAIRS(2,g),2)];
      
      subplot(NBXG,NBYG,g), plot2d(DATA(:,PAIRS(1,g)),DATA(:,PAIRS(2,g)),style=-9,rect=R);
          
          p=get("hdl");p.children(1).mark_size_unit='point';
          p.children(1).mark_size=PSIZE;
          TITLE = 'x= '+LEG(PAIRS(1,g))+'  y= '+LEG(PAIRS(2,g));
          xtitle(TITLE);

          x=DATA(:,PAIRS(1,g));y=DATA(:,PAIRS(2,g));
          
          // Scale
          gr=gca(); 
          flx = part(SCL(g),1);fly=part(SCL(g),2);
          xt = gr.x_ticks;yt=gr.y_ticks;
          
          if find(x<=0)<>[] & flx=='l' then, ..
            error('LOG scale is not possible for variable = '+LEG(PAIRS(1,g)));end
          if find(y<=0)<>[] & fly=='l' then, ..
            error('LOG scale is not possible for variable = '+LEG(PAIRS(2,g)));end
            
          if flx=='l' then
              iOK = find(xt(2)>0);xt(2) = xt(2)(iOK);xt(3)=xt(3)(iOK);
          end
          if fly=='l' then
              iOK = find(yt(2)>0);yt(2) = yt(2)(iOK);yt(3)=yt(3)(iOK);
          end
          gr.x_ticks=xt;gr.y_ticks=yt;
          gr.log_flags = flx+fly;
          gr.sub_ticks=[0,0];

          // Labels
          if okLab then, for pp=1:size(x,1),xstring(x(pp),y(pp),LAB(pp));end; end;
  end

endfunction






