function [X,CR,varargout]=GR_CAL_LM(Q,P,E,TS,GRname,varargin)
//******************************************************************
//  Function to calibrate the GRR4J model with a split sample test
//
//  Input arguments of the function :
//        Q     Discharge data (matrix [Nx1] where N is the number of time steps)
//        P     Rainfall data (matrix [Nx1] where N is the number of time steps)
//        E     Potential evapotranspiration data (matrix [Nx1] where N is the number of time steps)
//        TS    Time step length in hours
//        GRname Name of the GR model to use
//        {opt. param. 1}  ICAL   Indices of Q vector to be used for evaluation of the objective function in calibration period
//        {opt. param. 2}  TSini  Length of warming-up period
//        {opt. param. 3}  DEBFIN Start-End of period 1 and 2 (cross validation)
//        {opt. param. 4}  FOBJ   Transformation applied to calculate the objective function (default = 'Q', possible = 'logQ' or 'racQ')
//        {opt. param. 5}  XFIX   Fixed parameters in the calibration process
//        {opt. param. 6}  ICONT  Indices of Q vector to be used for evaluation of the objective function in control mode
//        {opt. param. 7}  Xini   List of parameters to be tested in the prefiltering operation  (list object of 5 vectors, each vector contain the different values 
//                                to be tested for each parameter
//
//  Output arguments of the function :
//        X     2 sets of parameters corresponding to calibration on P1 and P2
//        CR    Quality criteria (matrix [4x5], col1 = cal on P1, col2 = cal on P2, col3 = cont on P1, col3 = cont on P2)
//        {opt. out. 1}   CRlog Quality criteria on log values (matrix [4x5], col1 = cal on P1, col2 = cal on P2, col3 = cont on P1, col3 = cont on P2)
//        {opt. out. 2}   CRrac Quality criteria (on square root values (matrix [4x5], col1 = cal on P1, col2 = cal on P2, col3 = cont on P1, col3 = cont on P2)
//        {opt. out. 3}   CALC  Calculated outputs in calibration and validation (matrix [Nx2], col1 = calibration, col2=validation)
//        {opt. out. 4}   S     Calculated time series of S levels (matrix [Nx2], col1 = calibration, col2=validation)
//        {opt. out. 5}   R     Calculated time series of R levels (matrix [Nx2], col1 = calibration, col2=validation)
//
//------ Validation of input variables -------
    if find(GRname==['GR4J' 'GR2M' 'GR3M'])==[]  then,  error('Wrong GR model name'); end;
    if size(Q,1)<>size(P,1) | size(Q,1)<>size(E,1)  then,  error('Wrong size of Q, P or E vector'); end;
    if size(Q,2)<>1 | size(P,2)<>1 | size(E,2)<>1   then,  error('Wrong size of Q, P or E vector'); end;    
    if find(isnan(P))<>[] | find(isnan(E))<>[] | find(isnan(Q))<>[] then,  error('Nan values in Q, P or E vector'); end;    

    if find(Q<0)<>[] then, warning(msprintf('%3.1f',size(find(Q<0),'*')/size(Q,'*')*100) + ...
        '% of negative values in Q'); end;    
    if find(P<0)<>[] then, warning(msprintf('%3.1f',size(find(P<0),'*')/size(Q,'*')*100) + ...
        '% of negative values in P'); end;    
    if find(E<0)<>[] then, warning(msprintf('%3.1f',size(find(E<0),'*')/size(Q,'*')*100) + ...
        '% of negative values in E'); end;    
    
    Nval  = size(Q,1);    

//------ Optional parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined
    NBARGFIX = 5;

    ICAL = find(Q>=0);      
    if rhs >= NBARGFIX+1 then, if varargin(1)<>'def' then,ICAL  = varargin(1);end; end;
    if min(ICAL)<1 | max(ICAL)>size(Q,1) then error('ICAL has wrong values'); end;
    if find(Q(ICAL)<0)<>[] then
        error('Negative value of Q within the calibration sample'); 
    end
    
    TSini = 365*24/TS;      
    if rhs >= NBARGFIX+2 then, if varargin(2)<>'def' then,TSini  = varargin(2); end;end;
    if TSini<>365*24/TS then, warning('TSini is different from 1 year, TSini=' + ... 
        msprintf('%3.1f',TSini*TS/24/365)+ ' year(s)'); end;

    DEBFIN = [[1;int((Nval+TSini)/2)] [int((Nval-TSini)/2);Nval]];       
    if rhs >= NBARGFIX+3 then, if varargin(3)<>'def' then,DEBFIN  = varargin(3); end;end;
    if size(DEBFIN,1)<>2 & size(DEBFIN,2)<>2 then, error('Wrong size of DEBFIN'); end;
    if DEBFIN(1,1)<1 | DEBFIN(1,2)<1 then, error('Values inferior to 1 in DEBFIN'); end;
    if DEBFIN(2,1)>size(Q,1) | DEBFIN(2,2)>size(Q,1) then, error('Values larger than the length of Q in DEBFIN'); end;

    FOBJ  = 'Q';            
    if rhs >= NBARGFIX+4 then, if varargin(4)<>'def' then,FOBJ= varargin(4); end;end;
    if FOBJ<>'Q' & FOBJ<>'racQ' & FOBJ<>'logQ' then, error('Wrong objective function'); end
    if FOBJ=='Q'    then, deff('[TransQ]=QTRANS(Q)','TransQ=Q'); end
    if FOBJ=='racQ' then, deff('[TransQ]=QTRANS(Q)','TransQ=max(Q,0).^0.5'); end
    if FOBJ=='logQ' then, deff('[TransQ]=QTRANS(Q)','TransQ=log(max(Q,0.0001))'); end
    
    select GRname
        case 'GR4J' then XFIX = [%nan %nan %nan %nan 0]';XFIX(:,2)=XFIX;
        case 'GR2M' then XFIX = [%nan %nan 60 0 0]';XFIX(:,2)=XFIX;
        case 'GR3M' then XFIX = [%nan %nan 60 0 %nan]';XFIX(:,2)=XFIX;
    end
    if rhs >= NBARGFIX+5 then, if varargin(5)<>'def' then,XFIX= varargin(5); end;end;
    if size(XFIX,1)<>5 | (size(XFIX,2)<>2 & size(XFIX,2)<>1) then, error('Wrong size of XFIX'); end;
    if size(XFIX,2)==1 then, XFIX=[XFIX,XFIX]; end; // Same values attributed to both periods
    Ifix = find(~isnan(XFIX(:,1)));

    ICONT = ICAL;
    if rhs >= NBARGFIX+6 then, if varargin(6)<>'def' then,ICONT= varargin(6); end;end;
    if min(ICONT)<1 | max(ICONT)>size(Q,1) then error('IONT has wrong values'); end;
    if find(Q(ICONT)<0)<>[] then
        error('Negative value of Q within the control sample'); 
    end

    // initial values
    select GRname
        case 'GR4J' then Xini_GR  = list([250,370,700],[-3.7,-2,-0.8],[44,73,137],[1.5,1.7,2.1]*24/TS,0);
        case 'GR2M' then Xini_GR  = list([250,370,700],[0.5,1,1.5],60,0,0);
        case 'GR3M' then Xini_GR  = list([250,370,700],[0.5,1,1.5],60,0,[-50 0 50]);
    end
    if rhs >= NBARGFIX+7 then, if varargin(7)<>'def' then,Xini_GR= varargin(7); end;end;
    
    // Initial reservoir level
    RSini=[];
    
    // Option of lsqrsolve function :
    // max iteration = 5000
    // min dSSE = 1.d-4
    LSQRSOLVEoptions = [1.d-4,1.d-8,1.d-5,5000,0,100];        
    
//------ Definition of sub periods -------
    
    // Attention, exploitation optimale des donnes >> 1/2 anne de recoupement des priodes
    // (dfinition par dfaut de DEBFIN)
    IP1     = DEBFIN(1,1):DEBFIN(2,1); 
    ICALP1  = ICAL(find(ICAL<=IP1($) & ICAL>=IP1(1)+TSini));
    ICONTP1 = ICONT(find(ICONT<=IP1($) & ICONT>=IP1(1)+TSini));
    
    IP2     = DEBFIN(1,2):DEBFIN(2,2);
    ICALP2  = ICAL(find(ICAL<=IP2($) & ICAL>=IP2(1)+TSini));
    ICONTP2 = ICONT(find(ICONT<=IP2($) & ICONT>=IP2(1)+TSini));
    
    if ICALP1==[] | ICALP2==[] then, error('ICALP1 or ICALP2 are empty !'); end;
    if ICONTP1==[] | ICONTP2==[] then, error('ICONTP1 or ICONTP2 are empty !'); end;
    
//------ Objective function -------
    // Parameters transformation
    //    Sens = 0 >> Xt 2 X
    //    Sens = 1 >> X 2 Xt
    function [Xout]=XTRANS(Xin,Sens)
        if Sens==0 then 
            Xout(1)=max(1,exp(Xin(1)));
            Xout(2)=sign(Xin(2))*min(100,abs(sinh(Xin(2))));
            Xout(3)=max(1,exp(Xin(3)));
            Xout(4)=max(0.5,6/TS*(Xin(4)+10));
            Xout(5)=sign(Xin(5))*min(300,abs(sinh(Xin(5)))); 
            Xout(Ifix)=XFIX(Ifix);
        end
        if Sens==1 then
            Xout(1)=log(max(Xin(1),0.0001));
            Xout(2)=asinh(Xin(2));
            Xout(3)=log(max(Xin(3),0.0001));
            Xout(4)=TS/6*Xin(4)-10;
            Xout(5)=asinh(Xin(5));
        end
    endfunction
  
    // Objective function
    fPar='\t\tIter.%3d NS=%5.2f  X=%8.1f %8.1f %8.1f %8.1f\n';
    function [OBJ]=FctObjGR_LM(Xt,m)
      global nIT;X=XTRANS(Xt,0);
      // Premire execution de GR pour verifier l'initialisation
          execstr('[Qsim,Ssim,Rsim]=c_'+GRname+'(TS,X,P,E,[0.5;0.5])');
          RSini2=RSini;
          if RSini2==[], RSini2=[mean(Ssim(365*24/TS:365*24/TS:$))/X(1);mean(Rsim(365*24/TS:365*24/TS:$))/X(3)];end;
      // Deuxime excution de GR pour calculer le critre
          execstr('Qsim=c_'+GRname+'(TS,X,Pcal,Ecal,RSini2);');
          OBJ=QTRANS(Qsim(IcalageGR-DEBpercal+1))-QTRANS(Qcal(IcalageGR-DEBpercal+1));
      // Affichage
          if modulo(nIT,freqAff)==0 
            cr=CRIT(QTRANS(Qcal(IcalageGR-DEBpercal+1)),QTRANS(Qsim(IcalageGR-DEBpercal+1)));
            mprintf(fPar,nIT,cr(2),X(1:4)');
          end
       // Incrment du nb d'itration 
          nIT=nIT+1;
    endfunction
//     'if nIT==30 then, pause; end;'+...
    
//------ Preparation of the parameters pre filtering operation -------

    // List of combinations
    listX   = COMBIN(Xini_GR);
    // Initialisation of minimal SSE
    minSSE  = [%inf %inf];

//------ MAIN PART OF THE FUNCTION : Loop over the sub-periods and validation -------
    CALC = -9.99 * ones(Nval,2);
    S = -9.99 * ones(Nval,2);
    R = -9.99 * ones(Nval,2);
    ECH = -9.99 * ones(Nval,2);
    
    // Inputs
    INP = ['Q' 'P' 'E'];

    // iteration count
    global nIT;
    
    for per = 1:2
        mprintf('\tPeriod %d\n',per);
        
        // Data for the period
        execstr('Ipercal = IP'+string(per)+';');            // Sub-period
        DEBpercal = Ipercal(1);
        execstr('IcalageGR = ICALP'+string(per)+';');     // Calibration within sub-period
        execstr('Ipercont = IP'+string(3-per)+';');         // Sub-period
        DEBpercont = Ipercont(1);        
        execstr('IcontroleGR = ICONTP'+string(3-per)+';'); // Control from the other sub-period

        // Inputs
        for inp=1:size(INP,'*'), execstr(INP(inp)+'cal='+INP(inp)+'(Ipercal);'+ ...
                    INP(inp)+'cont='+INP(inp)+'(Ipercont);'); end;

        // Determination of the optimal set of initial parameters
        mprintf('\t\tTest of %d parameter sets\n',size(listX,2));        
        freqAff=size(listX,2)+1;nIT=1;
        for np=1:size(listX,2)
            Xt  = XTRANS(listX(:,np),1);
            SSE = sum((FctObjGR_LM(Xt,0)).^2);
            if SSE<minSSE(per) then
                minSSE(per)  = SSE;
                Xtini(:,per) = Xt;
            end
        end

        //Calibration
        freqAff=10;nIT=1;
        Xtemp  = lsqrsolve(Xtini(:,per),FctObjGR_LM,size(IcalageGR,2),LSQRSOLVEoptions);
        X(:,per) = XTRANS(Xtemp,0);

        // initialisation
        execstr('[REStemp,Stemp,Rtemp] = c_'+GRname+'(TS,X(:,per),P,E,[0.5;0.5]);');                
        RSini2=RSini;if RSini2==[], RSini2=[mean(Stemp(Nval:365*24/TS:$))/X(1,per);mean(Rtemp(Nval:365*24/TS:$))/X(3,per)];end;
        
        // Storage of results in calibration mode - Simulation on Ipercal
        execstr('[REStemp,Stemp,Rtemp,ECHtemp] = c_'+GRname+'(TS,X(:,per),Pcal,Ecal,RSini2);');
        CALC(Ipercal(TSini:$),1)   = REStemp(TSini:$);
        S(Ipercal(TSini:$),1)      = Stemp(TSini:$);
        R(Ipercal(TSini:$),1)      = Rtemp(TSini:$);
        ECH(Ipercal(TSini:$),1)    = ECHtemp(TSini:$);

        // Criteria in calibration mode - Simulation on Ipercal
        CR(:,per) = CRIT(Qcal(IcalageGR-DEBpercal+1),REStemp(IcalageGR-DEBpercal+1));
        CRlog(:,per) = CRIT(log(max(Qcal(IcalageGR-DEBpercal+1),0.0001)),log(max(REStemp(IcalageGR-DEBpercal+1),0.0001)));
        CRrac(:,per) = CRIT(sqrt(max(Qcal(IcalageGR-DEBpercal+1),0)),sqrt(max(REStemp(IcalageGR-DEBpercal+1),0)));

        // Storage of results in validation mode - Simulation on Ipercont
        execstr('[REStemp,Stemp,Rtemp,ECHtemp] = c_'+GRname+'(TS,X(:,per),Pcont,Econt,RSini2);');
        CALC(Ipercont(TSini:$),2)   = REStemp(TSini:$);
        S(Ipercont(TSini:$),2)      = Stemp(TSini:$);
        R(Ipercont(TSini:$),2)      = Rtemp(TSini:$);
        ECH(Ipercont(TSini:$),2)    = ECHtemp(TSini:$);

        // Criteria in validation mode - Simulation on Ipercont
        CR(:,3-per+2) = CRIT(Qcont(IcontroleGR-DEBpercont+1),REStemp(IcontroleGR-DEBpercont+1));
        CRlog(:,3-per+2) = CRIT(log(max(Qcont(IcontroleGR-DEBpercont+1),0.0001)),log(max(REStemp(IcontroleGR-DEBpercont+1),0.0001)));
        CRrac(:,3-per+2) = CRIT(sqrt(max(Qcont(IcontroleGR-DEBpercont+1),0)),sqrt(max(REStemp(IcontroleGR-DEBpercont+1),0)));
    end

//------ Outputs -------
    if lhs>=3 then, varargout(1)=CRlog; end
    if lhs>=4 then, varargout(2)=CRrac; end
    if lhs>=5 then, varargout(3)=CALC; end
    if lhs>=6 then, varargout(4)=S; end
    if lhs>=7 then, varargout(5)=R; end
    if lhs>=8 then, varargout(6)=ECH; end
    
endfunction
