function [QC,varargout]=GR_SIM(X,P,E,TS,GRname,varargin)
//******************************************************************
//  Function to generate a simulation with a GR model
//
//  Input arguments of the function :
//        X     1 sets of parameters to be used
//        
//        P     Rainfall data (matrix [Nx1] where N is the number of time steps)
//        E     Potential evapotranspiration data (matrix [Nx1] where N is the number of time steps)
//        TS    Time step length in hours
//        GRname Name of the GR model to use
//        {opt. param. 1}  TSini  Length of warming-up period
//        {opt. param. 2}  Q      Discharge data (matrix [Nx1] where N is the number of time steps)
//        {opt. param. 3}  Icrit  Index of values where the criteria should be calculated
//
//  Output arguments of the function :
//        QC  Calculated outputs in calibration and validation (matrix [Nx1])
//        {opt. out. 1}   S     Calculated time series of S levels (matrix [Nx1])
//        {opt. out. 2}   R     Calculated time series of R levels (matrix [Nx1])
//        {opt. out. 3}   ECH   Calculated time series of IGF (matrix [Nx1])
//        {opt. out. 4}   ER    Calculated time series of real evapotranspiration (matrix [Nx1])
//        {opt. out. 5}   PR    Calculated time series of effective rainfall (matrix [Nx1])
//        {opt. out. 6}   CR    Quality criteria (matrix [4x1])
//        {opt. out. 7}   CRlog Quality criteria on log values (matrix [4x1])
//        {opt. out. 8}   CRrac Quality criteria on square root values (matrix [4x1])
//
//------ Validation of input variables -------

    if find(GRname==['GR4J' 'GR2M' 'GR3M'])==[]  then,  error('Wrong GR model name'); end;
    if size(P,1)<>size(E,1)  then,  error('Wrong size of P or E vector'); end;
    if find(isnan(P))<>[] | find(isnan(E))<>[] then,  error('Nan values in P or E vector'); end;    

    if find(P<0)<>[] then, warning('     '+msprintf('%3.1f',size(find(P<0),'*')/size(P,'*')*100) + ...
        '% of negative values in P'); end;    
    if find(E<0)<>[] then, warning('     '+msprintf('%3.1f',size(find(E<0),'*')/size(P,'*')*100) + ...
        '% of negative values in E'); end;    
    
    Nval  = size(P,1);    

//------ Optional parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined
    NBARGFIX = 5;
    
    TSini = 365*24/TS;      
    if rhs >= NBARGFIX+1 then, if varargin(1)<>'def' then,TSini  = varargin(1); end;end;
    if TSini<>365*24/TS then, warning('TSini is different from 1 year, TSini=' + ... 
        msprintf('%3.1f',TSini*TS/24/365)+ ' year(s)'); end;

    Q = [];      
    if rhs >= NBARGFIX+2 then, Q  = varargin(2);end;
    if size(Q,1)<>0 & size(Q,1)<>size(P,1)  then,  error('Wrong size of Q, P or E vector'); end;
    
    Icrit = find(Q>=0);      
    if rhs >= NBARGFIX+3 then, Icrit  = varargin(3);end;
    if find(Q(Icrit)<0)<>[] then, warning('     '+msprintf('%3.1f',size(find(Q(Icrit)<0),'*')/size(P,'*')*100) + ...
        '% of negative values in Q(Icrit)'); end;    

//------ MAIN PART OF THE FUNCTION : Loop over the sub-periods and validation -------

        // initialisation
        execstr('[QC,S,R,ECH,ER,PR] = c_'+GRname+'(TS,X,P,E,[0.5;0.5]);');                
        RSini=[mean(S(365*24/TS:365*24/TS:$))/X(1);mean(R(365*24/TS:365*24/TS:$))/X(3)];
        
        // Storage of results
        execstr('[QC,S,R,ECH,ER,PR]  = c_'+GRname+'(TS,X,P,E,RSini);');
 
        // Criteria
        CR=[];CRlog=[];CRrac=[];
        if Q<>[] & Icrit<>[] then
            CR = CRIT(Q(Icrit),QC(Icrit));
            CRlog = CRIT(log(max(Q(Icrit),0.0001)),log(max(QC(Icrit),0.0001)));
            CRrac = CRIT(sqrt(max(Q(Icrit),0)),sqrt(max(QC(Icrit),0)));
        end

//------ Outputs -------
    if lhs>=2 then, varargout(1)=S; end
    if lhs>=3 then, varargout(2)=R; end
    if lhs>=4 then, varargout(3)=ECH; end
    if lhs>=5 then, varargout(4)=ER; end
    if lhs>=6 then, varargout(5)=PR; end
    if lhs>=7 then, varargout(6)=CR; end
    if lhs>=8 then, varargout(7)=CRlog; end
    if lhs>=9 then, varargout(8)=CRrac; end
endfunction
