function [AJUST]=GUMB_AJUST(DATA,varargin)
//******************************************************************
//  Function to estimate return periods based on an adjustment 
//  of yearly maximal values on a Gumbel law
//
//  Input arguments of the function :
//        DATA       Data to estimate Gumbel law parameter, matrix [Nx1] with N = nb of years
//        {opt. param. 1}  GRAPH    Draw a graph or not (%t or %f, default = %t);
//        {opt. param. 2}  RETPER   Return period to calculate, matrix [Px1] with P=number of return period
//                                    default = [2;5;10;20;50;100];
//
//  Output arguments of the function :
//        FichGraph     List of generated files
//******************************************************************************************

//------ Validation of input variables -------
    if size(DATA,2)<>1  then, error('DATA must a vector'); end;

//------ Optional parameters -------
    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined
    NBARGFIX = 1;

    GRAPH= %t; if rhs >= NBARGFIX+1 then, if varargin(1)<>'def' then,GRAPH=varargin(1);end; end;

    RETPER=[2;5;10;20;50;100]; if rhs >= NBARGFIX+2 then, if varargin(2)<>'def' then,RETPER=varargin(2);end; end;
    if size(RETPER,2)<>1  then, error('RETPER must a vector'); end; 
    if find(RETPER<=0)<>[]  then, error('RETPER cannot contain negative or null values'); end; 
    
//------ MAIN PART OF THE FUNCTION : Loop over the sub-periods and validation -------

    gam=0.5772; // Euler constant
    
    // Empirical frequencies
    Fre = 1/(size(DATA,1)+0.4)*((1:size(DATA,1))'-0.3);
    uFre = -log(-log(Fre));
    
    // Sorted valuesin increasing order
    SORT = gsort(DATA,'r','i');
    
    // Gumbel law parameter
    b=sqrt(6)/%pi*st_deviation(DATA);
    a=mean(DATA)-b*gam;

    // Loop over the return periods
    AJUST(:,1)=RETPER;
    for p=1:size(RETPER,1)
        U = -log(-log(1-1/RETPER(p))); // Normalised gumbel variable
        AJUST(p,2) = a+b*U;
    end    
    
    // Graph
    FreCarac = 1-(RETPER).^(-1);
    uFreCarac = -log(-log(FreCarac));

    if GRAPH then
        // Limites du graph
        R = [-1,min(DATA),max(uFreCarac)*1.1,max(AJUST(:,2))*1.1];
        
        // Graph des donnes et de la loi de Gumbel
        clf();
        plot2d(uFre,a+b*uFre,style=2,rect=R);p=get("hdl");p.children(1).thickness=2;
        fig=gca();fig.font_size=3;fig.x_label.font_size=3;
        fig.y_label.font_size=3;
        plot2d(uFre,SORT,style=-9,rect=R);
        plot2d([uFre($) uFreCarac($)],[a+b*uFre($) AJUST($,2)],style=2,rect=R);
        p=get("hdl");p.children(1).thickness=2;p.children(1).line_style=2;
        xstring(3,R(2),'U, Gumbel variable');        
        
        // Axe repre
        yaxe = min(DATA)+(max(DATA)-min(DATA))*0.25;
        plot2d([uFreCarac(1) uFreCarac($)],yaxe*ones(1,2));
        for p=1:size(RETPER,1)
            plot2d(uFreCarac(p)*ones(1,2),yaxe+[(R(4)-R(2))/100 -(R(4)-R(2))/100]);
            xstring(uFreCarac(p),yaxe+(R(4)-R(2))/100,'T'+string(RETPER(p)));
        end
        
    end
endfunction
