function [X,CR,varargout]=PROPAG_CAL_LM(Qdo,Qup,TS,PRopt,varargin)
//******************************************************************
//  Function to calibrate a propagation model with a split sample test
//
//  Input arguments of the function :
//        Qdo   Downstream discharge data (matrix [Nx1] where N is the number of time steps)
//        Qup   Upstream discharge data (matrix [Nx1] where N is the number of time steps)
//        TS    Time step length in hours
//        PRopt Options in the propagation model used (integer)
//        {opt. param. 1}  ICAL   Indices of Q vector to use for evaluation of objective function
//        {opt. param. 2}  TSini  Length of warming-up period
//        {opt. param. 3}  DEBFIN Start-End of period 1 and 2 (cross validation)
//        {opt. param. 4}  FOBJ   Transformation applied to calculate the objective function (default = 'Q', possible = 'logQ' or 'racQ')
//        {opt. param. 5}  XFIX   Fixed parameters in the calibration process
//        {opt. param. 6}  ICONT  Indices of Q vector to use for evaluation of objective function in control mode
//
//  Output arguments of the function :
//        X     2 sets of parameters corresponding to calibration on P1 and P2
//        CR    Quality criteria (matrix [4x5], col1 = cal on P1, col2 = cal on P2, col3 = cont on P1, col3 = cont on P2)
//        {opt. out. 1}   CRlog Quality criteria on log values 
//                          (matrix [4x5], col1 = cal on P1, col2 = cal on P2, col3 = cont on P1, col3 = cont on P2)
//        {opt. out. 2}   CRrac Quality criteria (on square root values 
//                          (matrix [4x5], col1 = cal on P1, col2 = cal on P2, col3 = cont on P1, col3 = cont on P2)
//        {opt. out. 3}   CALC  Calculated outputs in calibration and validation 
//                          (matrix [Nx2], col1 = calibration, col2=validation)
//
//------ Validation of input variables -------
    if find(PRopt==[10+(0:2) 20+(0:2)])==[]  then,  error('Wrong propagation model option'); end;
    if size(Qdo,1)<>size(Qup,1) then,  error('Qdo and Qup vectors do not have the same number of rows'); end;
    if size(Qdo,2)<>1 | size(Qup,2)<>1 then,  error('Qdo and Qup vectors are not column vectors'); end;
    if find(isnan(Qdo))<>[] | find(isnan(Qup))<>[] then,  error('Nan values in Qup or Qdo vector'); end;    

    if find(Qdo<0)<>[] then, warning('     '+msprintf('%3.1f',size(find(Qdo<0),'*')/size(Qdo,'*')*100) + ...
        '% of negative values in Qdo'); end;    
    if find(Qup<0)<>[] then, warning('     '+msprintf('%3.1f',size(find(Qup<0),'*')/size(Qup,'*')*100) + ...
        '% of negative values in Qup'); end;    
    
    Nval  = size(Qup,1);    

//------ Optional parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined
    NBARGFIX = 4;

    ICAL = find(Qup>=0 & Qdo>=0);      
    if rhs >= NBARGFIX+1 then, if varargin(1)<>'def' then,ICAL  = varargin(1);end; end;
    if min(ICAL)<1 | max(ICAL)>size(Qup,1) then error('ICAL has wrong values'); end;
    if find(Qdo(ICAL)<0)<>[] then
        error('Negative value of Qdo within the calibration sample'); 
    end

    TSini = 30*24/TS;      
    if rhs >= NBARGFIX+2 then, if varargin(2)<>'def' then,TSini  = varargin(2); end;end;
    if TSini<>30*24/TS then, warning('TSini is different from 1 month, TSini=' + ... 
        msprintf('%3.1f',TSini*TS/24/30)+ ' month(s)'); end;

    DEBFIN = [[1;int((Nval+TSini)/2)] [int((Nval-TSini)/2);Nval]];       
    if rhs >= NBARGFIX+3 then, if varargin(3)<>'def' then,DEBFIN  = varargin(3); end;end;
    if size(DEBFIN,1)<>2 & size(DEBFIN,2)<>2 then, error('Wrong size of DEBFIN'); end;
    if DEBFIN(1,1)<1 | DEBFIN(1,2)<1 then, error('Values inferior to 1 in DEBFIN'); end;
    if DEBFIN(2,1)>size(Qup,1) | DEBFIN(2,2)>size(Qup,1) then, error('Values larger than the length of Q in DEBFIN'); end;

    FOBJ  = 'Q';            
    if rhs >= NBARGFIX+4 then, if varargin(4)<>'def' then,FOBJ= varargin(4); end;end;
    if FOBJ<>'Q' & FOBJ<>'racQ' & FOBJ<>'logQ' then, error('Wrong objective function'); end

    XFIX = [%nan %nan %nan %nan]'; // Rescaling factor set to 1 >> conservative model
    if rhs >= NBARGFIX+5 then, if varargin(5)<>'def' then,XFIX= varargin(5); end;end;
    if size(XFIX,1)<>4 | size(XFIX,2)<>1 then, error('Wrong size of XFIX'); end;
    Ifix = find(~isnan(XFIX));

    ICONT = ICAL;
    if rhs >= NBARGFIX+6 then, if varargin(6)<>'def' then,ICONT= varargin(6); end;end;
    if min(ICONT)<1 | max(ICONT)>size(Qdo,1) then error('IONT has wrong values'); end;
    if find(Qdo(ICONT)<0)<>[] then
        error('Negative value of Qdo within the control sample'); 
    end

    // initial values depending on the type of routing model
    Xini_PR  = list([0,0.5,2]*24/TS,[0.5,1,5],[0,2,10],1); // Other models than non-linear rescaling

//------ Definition of sub periods -------
    
    // Attention, exploitation optimale des donnes >> 1/2 anne de recoupement des priodes
    IP1     = DEBFIN(1,1):DEBFIN(2,1); 
    ICALP1  = ICAL(find(ICAL<=IP1($) & ICAL>=IP1(1)+TSini));
    ICONTP1 = ICONT(find(ICONT<=IP1($) & ICONT>=IP1(1)+TSini));
    
    IP2     = DEBFIN(1,2):DEBFIN(2,2);
    ICALP2  = ICAL(find(ICAL<=IP2($) & ICAL>=IP2(1)+TSini));
    ICONTP2 = ICONT(find(ICONT<=IP2($) & ICONT>=IP2(1)+TSini));
    
    if ICALP1==[] | ICALP2==[] then, error('ICALP1 or ICALP2 are empty !'); end;
    if ICONTP1==[] | ICONTP2==[] then, error('ICONTP1 or ICONTP2 are empty !'); end;

//------ Objective function -------
    // Parameters transformation
    //    Sens = 0 >> Xt 2 X
    //    Sens = 1 >> X 2 Xt
    deff('[Xout]=XTRANS(Xin,Sens)',...
    'if Sens==0 then, Xout([1,2,3])=exp(Xin([1,2,3]));Xout(4)=3/18*(Xin(3)+9);'+...
            'Xout=min(10000,max(0,Xout)); if PRopt-int(PRopt/10)*10<>3 then, Xout(4)=0; end;'+...
            ';Xout(Ifix)=XFIX(Ifix);end;'+...
    'if Sens==1 then, Xout([1,2,3])=log(max(Xin([1,2,3]),0.0001));Xout(4)=18/3*Xin(3)-9;end;')

    if FOBJ=='Q'    then, deff('[TransQ]=QTRANS(Q)','TransQ=Q'); end
    if FOBJ=='racQ' then, deff('[TransQ]=QTRANS(Q)','TransQ=max(Q,0).^0.5'); end
    if FOBJ=='logQ' then, deff('[TransQ]=QTRANS(Q)','TransQ=log(max(Q,0.0001))'); end
        
    deff('[OBJ]=FctObjPR_LM(Xt,m)',...
     'X=XTRANS(Xt,0);' + ...
     'QdoSim=c_PROPAG('+string(PRopt)+',X,QupCal);'+...
     'OBJ=QTRANS(QdoSim(IcalagePR-DEBpercal+1))-QTRANS(QdoCal(IcalagePR-DEBpercal+1));');
    
//------ Preparation of the parameters pre filtering operation -------

    // List of combinations
    listX   = COMBIN(Xini_PR);
    // Initialisation of minimal SSE
    minSSE  = [%inf %inf];

//------ MAIN PART OF THE FUNCTION : Loop over the sub-periods and validation -------
    CALC = -9.99 * ones(Nval,2);
    S = -9.99 * ones(Nval,2);
    R = -9.99 * ones(Nval,2);

    for per = 1:2

        // Data for the period
        execstr('Ipercal = IP'+string(per)+';');            // Sub-period
        DEBpercal = Ipercal(1);
        execstr('IcalagePR = ICALP'+string(per)+';');     // Calibration within sub-period
        execstr('Ipercont = IP'+string(3-per)+';');         // Sub-period
        DEBpercont = Ipercont(1);        
        execstr('IcontrolePR = ICONTP'+string(3-per)+';'); // Control from the other sub-period

        // Inputs
        QupCal=Qup(Ipercal);QupCont=Qup(Ipercont);
        QdoCal=Qdo(Ipercal);QdoCont=Qdo(Ipercont);

        // Determination of the optimal set of initial parameters
        for np=1:size(listX,2)
        Xt  = XTRANS(listX(:,np),1);SSE = sum((FctObjPR_LM(Xt,0)).^2);
            if SSE<minSSE(per) then, minSSE(per)  = SSE;Xtini(:,per) = Xt; end
        end

        //Calibration
        Xtemp   = lsqrsolve(Xtini(:,per),FctObjPR_LM,size(IcalagePR,2));
        X(:,per)= XTRANS(Xtemp,0);

        // Storage of results in calibration mode - Simulation on Ipercal
        execstr('QdoSimCal = c_PROPAG('+string(PRopt)+',X(:,per),QupCal);');
        CALC(Ipercal(TSini:$),1)   = QdoSimCal(TSini:$);

        // Criteria in calibration mode - Simulation on Ipercal
        CR(:,per) = CRIT(QdoCal(IcalagePR-DEBpercal+1),QdoSimCal(IcalagePR-DEBpercal+1));
        CRlog(:,per) = CRIT(log(max(QdoCal(IcalagePR-DEBpercal+1),0.0001)),log(max(QdoSimCal(IcalagePR-DEBpercal+1),0.0001)));
        CRrac(:,per) = CRIT(sqrt(max(QdoCal(IcalagePR-DEBpercal+1),0)),sqrt(max(QdoSimCal(IcalagePR-DEBpercal+1),0)));

        // Storage of results in validation mode - Simulation on Ipercont
        execstr('QdoSimCont = c_PROPAG('+string(PRopt)+',X(:,per),QupCont);');
        CALC(Ipercont(TSini:$),2) = QdoSimCont(TSini:$);

        // Criteria in validation mode - Simulation on Ipercont
        CR(:,3-per+2) = CRIT(QdoCont(IcontrolePR-DEBpercont+1),QdoSimCont(IcontrolePR-DEBpercont+1));
        CRlog(:,3-per+2) = CRIT(log(max(QdoCont(IcontrolePR-DEBpercont+1),0.0001)),log(max(QdoSimCont(IcontrolePR-DEBpercont+1),0.0001)));
        CRrac(:,3-per+2) = CRIT(sqrt(max(QdoCont(IcontrolePR-DEBpercont+1),0)),sqrt(max(QdoSimCont(IcontrolePR-DEBpercont+1),0)));
        
    end

//------ Outputs -------
    if lhs>=3 then, varargout(1)=CRlog; end
    if lhs>=4 then, varargout(2)=CRrac; end
    if lhs>=5 then, varargout(3)=CALC; end
    
endfunction
