function OUT=TARAGE(CTAB,DATA,varargin)
//******************************************************************
//  Function to convert data with the help of a tabulated conversion
//  table. This function is designed to convert discharges in water levels and
//  vice versa with a tabulated rating curve.
//
//  Input arguments of the function :
//        CTAB  Conversion data table (matrix [Px2] where P is the number of correspondance points)
//        IN    Data to convert (matrix [Nx1] where N is the number of data)
//        TARG  Target data (matrix [Mx1] where N is the number of data)
//        {opt. param. 1}  TYPE         Type of conversion (default = 0): 
//                                        'left2right'=DATA is expressed according to the first column of CTAB (hence OUT -> second)
//                                        'right2left'=DATA is expressed according to the second column of CTAB  (hence OUT -> first)
//        {opt. param. 2}  ExtrapolInf  Type of extrapolation for values greater than the max of CTAB
//                                        'Nul'=Nan value affected
//                                        'Lin'=Linear extrapolation on the basis of the 2 last values
//                                        'Cst'=Extrapolation is constant and equal to the greater value in CTAB
//        {opt. param. 3}  ExtrapolSup  Type of extrapolation for values smaller than the min of CTAB
//                                        'Nul'=Nan value affected
//                                        'Lin'=Linear extrapolation on the basis of the 2 last values
//                                        'Cst'=Extrapolation is constant and equal to the greater value in CTAB//
//
//        {opt. param. 4}  NbPtSup      Number of points used to extrapolate the highest values (default = 3)
//
//
//  Output arguments of the function :
//        OUT   Converted data (Matrix [Nx1] where N is the number of data)

//------ Validation of input variables -------
    if size(DATA,2)<>1 then, error('DATA should be a column vector'); end;
    if size(CTAB,2)<>2 then, error('CTAB should have 2 columns'); end;

//------ Optional parameters -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined
    
    TYPE = 'left2right';             if rhs >= 3 then, if varargin(1)<>'def' then, TYPE  = varargin(1); end; end;
    if TYPE<>'left2right' & TYPE <>'right2left' then, error('Wrong TYPE value'); end;
    
    ExtrapolInf = 'Cst';  if rhs >= 4 then, if varargin(2)<>'def' then, ExtrapolInf  = varargin(2); end; end;
    if ~((ExtrapolInf=='Lin')|(ExtrapolInf=='Cst')|(ExtrapolInf=='Nul')) then, error('Wrong ExtrapolInf value'); end;

    ExtrapolSup = 'Lin';  if rhs >= 5 then, if varargin(3)<>'def' then, ExtrapolSup  = varargin(3); end; end;
    if ~((ExtrapolSup=='Lin')|(ExtrapolSup=='Cst')|(ExtrapolSup=='Nul')) then, error('Wrong ExtrapolSup value'); end;

    NbPtSup = 3;  if rhs >= 5 then, if varargin(4)<>'def' then, NbPtSup  = varargin(4); end; end;
    if ~((ExtrapolSup=='Lin')|(ExtrapolSup=='Cst')|(ExtrapolSup=='Nul')) then, error('Wrong ExtrapolSup value'); end;

//------ CORE FUNCTION CODE -------

    // Sorting of CONVTAB
        if TYPE=='right2left' then,  CTAB=CTAB(:,[2,1]); end;
        [Croiss,k]  = gsort(CTAB(:,1),'r','i');CTtemp=[];CTtemp(:,1)=Croiss;CTtemp(:,2)=CTAB(k,2);
        
        // correction of decreasing values in CT(:,2)
        if find(diff(CTtemp(:,2))<=0)<>[] then
            Compte=0;CT=[];
            for n=2:size(CTtemp,1)
                if CTtemp(n,2)>max(CTtemp(1:n-1,2)) then,
                   CT = [CT;CTtemp(n,:)];
                else
                    Compte=Compte+1;
                end
            end
            if Compte>0 then, ...
               // warning(string(Compte) + ' wrong values in col2 of CTAB (decreasing values). These values are ignored'); 
               end;
        else
            CT=CTtemp;
        end

    // Linear conversion of data if they fall into the interval [min(CTAB(:,1)) .. max(CTAB(:,1))]
        OUT= linear_interpn(DATA,CT(:,1),CT(:,2));

    // Conversion of data inferior to min(CTAB(:,1))
        Iinf    = find(DATA<min(CT(:,1)));
        x1=CT(1,1);y1=CT(1,2);x2=CT(2,1);y2=CT(2,2);
        if ExtrapolInf=="Lin" then
          OUT(Iinf)=(y2-y1)/(x2-x1).*(DATA(Iinf)-x1)+y1;
        elseif ExtrapolInf=="Nul" then
          OUT(Iinf)=%nan;
        elseif ExtrapolInf=="Cst" then
          OUT(Iinf)=CT(1,2);
        end

    // Conversion of data superior to max(CTAB(:,1))
        Isup=find(DATA>max(CT(:,1)));
        [a,b,sig] = reglin(CT($-NbPtSup:$,1)',CT($-NbPtSup:$,2)');

        if ExtrapolSup=="Lin" then
          OUT(Isup)=DATA(Isup)*a+b;
        elseif ExtrapolSup=="Nul" then
          OUT(Isup)=%nan;
        elseif ExtrapolSup=="Cst" then
          OUT(Isup)=CT($,2);
        end

endfunction

